-- This file is part of the YateAdmin Project http://yateadmin.null.ro
-- A free web interface to set up a server based on the Yate telephony engine.
-- Copyright (C) 2006 Null Team
--
-- Yet Another Telephony Engine - a fully featured software PBX and IVR
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

--
-- PostgreSQL database dump
--

SET client_encoding = 'LATIN1';
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public schema';


SET search_path = public, pg_catalog;

--
-- Name: plpgsql_call_handler(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


ALTER FUNCTION public.plpgsql_call_handler() OWNER TO postgres;

--
-- Name: plpgsql_validator(oid); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_validator(oid) RETURNS void
    AS '$libdir/plpgsql', 'plpgsql_validator'
    LANGUAGE c;


ALTER FUNCTION public.plpgsql_validator(oid) OWNER TO postgres;

--
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: public; Owner:
--

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler VALIDATOR plpgsql_validator;


--
-- Name: getroute(text, text); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION getroute(text, text) RETURNS SETOF record
    AS $_$
	DECLARE
		callednr ALIAS FOR $1;
		rtp_f ALIAS FOR $2;
		result record;
		partial2 record;
		iteration int2;
		iteration2 int2;
		iteration3 int2;
		newcalled text;
		loc record;
		dig text;
		flag text;
		locat text;
		format text;
		rtp text;
		div text;
		div_busy text;
		div_noanswer text;
		noans_timeout int2;
		ips text[];
		dim int2;
		block text;
		i int2;
		query text;
		div_info record;
		leavemail text;
	BEGIN
		iteration:=0;
		newcalled := callednr;
		SELECT INTO leavemail value FROM settings WHERE param='vm';
		IF NOT FOUND THEN
			raise notice 'leavemail is not set';
		END IF;

		FOR loc IN SELECT * FROM form LIMIT 1 LOOP
			SELECT into locat line FROM lines WHERE line=callednr;
			loc.location := NULL;
			IF FOUND THEN
				loc.formats:=NULL;
				SELECT into div,div_busy,div_noanswer,noans_timeout MAX(CASE WHEN param='forward' THEN value END),MAX(CASE WHEN param='forward_busy' THEN value END),MAX(CASE WHEN param='forward_noanswer' THEN value END) ,MAX(CASE WHEN param='noanswer_timeout' THEN value END)  FROM pbx WHERE line=callednr;
				IF FOUND THEN
					IF div != callednr AND div IS NOT NULL AND div != '' THEN
						newcalled := div;
					ELSE
						newcalled := callednr;
						loc.divert_busy := CASE WHEN (div_busy = 'vm') THEN leavemail ELSE div_busy END;
						loc.divert_noanswer := CASE WHEN (div_noanswer = 'vm') THEN leavemail ELSE div_noanswer END;
						IF div_noanswer IS NOT NULL AND div_noanswer != '' THEN
							loc.maxcall := noans_timeout*1000;
						END IF;
					END IF;
				ELSE
					newcalled := callednr;
				END IF;
				if newcalled != 'vm' THEN
					SELECT into locat location FROM lines WHERE line=newcalled;
					IF FOUND AND locat IS NOT NULL AND locat != '' THEN
						loc.location := locat;
						loc.called := newcalled;
						iteration := iteration + 1;
						RETURN next loc;
						RETURN;
					END IF;
				ELSE
					loc.location := leavemail;
					loc.called := callednr;
					iteration := iteration + 1;
					RETURN next loc;
					RETURN;
				END IF;
			END IF;
		END LOOP;
		iteration2:=0;
		dim := 1;
		FOR result IN SELECT * FROM form LIMIT 3 LOOP
			IF iteration>2 THEN
				EXIT;
			END IF;

			SELECT INTO dig prefix FROM routes WHERE prefix=substr(newcalled,1,LENGTH(prefix)) ORDER BY length(prefix) DESC LIMIT 1 OFFSET iteration2;
			IF NOT FOUND THEN
				IF iteration=0 THEN
					result.location:='';
					result.error:='unknown prefix';
					RETURN next result;
				END IF;
				RETURN;
				EXIT;
			END IF;
			iteration2:=iteration2+1;
			iteration3:=0;
			LOOP
				SELECT INTO partial2 * FROM routes WHERE prefix=dig OR prefix='' ORDER BY priority LIMIT 1 OFFSET iteration3;
				IF NOT FOUND THEN
					IF iteration=0 and iteration3=0 THEN
						result.error:='no route to call';
						result.called:=newcalled;
						RETURN next result;
					END IF;
					EXIT;
				END IF;

				result.rtp_forward:=NULL;
				result.formats:=NULL;
				block := 'no';
				IF dim > 1 THEN
					i := 1;
					LOOP
					--if the trnk is already in the already found trunks
						IF partial2.ip = ips[i] THEN
							block := 'yes';
						END IF;
						i := i+1;
						IF i+1 > dim THEN
							EXIT;
						END IF;
					END LOOP;
				END IF;
				IF block != 'yes' THEN
					newcalled := rewritenr(newcalled,partial2.cutbool,partial2.nrdigcut,partial2.poscut,partial2.replacebool,partial2.posrpl,partial2.digrpl,partial2.nrdigrpl);
					iteration:=iteration+1;
					IF partial2.account IS NOT NULL AND partial2.account!='' AND partial2.protocol<>'zap' AND partial2.protocol<>'wp' THEN
						result.location:= partial2.protocol || '/' || newcalled;
						result.line:=partial2.account;
						SELECT INTO format formats FROM accounts WHERE account=partial2.account;
						IF FOUND AND format IS NOT NULL AND format != '' THEN
							result.formats:=format;
						END IF;
						IF FOUND AND partial2.rtp_forward IS TRUE AND rtp_f='possible' THEN
							result.rtp_forward:='yes';
						END IF;
					ELSEIF partial2.protocol='sip' THEN
						IF partial2.port IS NULL OR partial2.port='' THEN
							result.location := 'sip/sip:' || newcalled || '@' || partial2.ip;
						ELSE
							result.location := 'sip/sip:' || newcalled || '@' || partial2.ip || ':' || partial2.port;
						END IF;
					ELSEIF partial2.protocol='h323' THEN
						IF partial2.port IS NULL OR partial2.port='' THEN
							result.location := 'h323/' || newcalled || '@' || partial2.ip;
						ELSE
							result.location := 'h323/' || newcalled || '@' || partial2.ip || ':' || partial2.port;
						END IF;
					ELSEIF partial2.protocol='wp' OR partial2.protocol='zap' THEN
						result.location := partial2.protocol || '/' || partial2.chan_group || '/' || newcalled;
					ELSEIF partial2.protocol='iax' THEN
						IF partial2.iaxuser IS NULL THEN
							partial2.iaxuser:='';
						END IF;
						IF partial2.iaxcontext IS NULL OR partial2.iaxcontext='' THEN
							IF partial2.port IS NULL OR partial2.port='' THEN
								result.location := 'iax/' || partial2.iaxuser || '@'|| partial2.ip || '/' || newcalled;
							ELSE
								result.location := 'iax/' || partial2.iaxuser || '@' || partial2.ip || ':' || partial2.port || '/' || newcalled;
							END IF;
						ELSE
							IF partial2.port IS NULL OR partial2.port='' THEN
								result.location := 'iax/' || partial2.iaxuser || '@' || partial2.ip || '/' || newcalled || '@' || partial2.iaxcontext;
							ELSE
								result.location := 'iax/' || partial2.iaxuser || '@' || partial2.ip || ':' || partial2.port || '/' || newcalled || '@' || partial2.iaxcontext;
							END IF;
						END IF;
					END IF;
					IF partial2.formats IS NOT NULL AND partial2.formats != '' AND (partial2.account IS NULL OR partial2.account = '') THEN
						result.formats := partial2.formats;
					END IF;
					IF partial2.rtp_forward IS TRUE AND rtp_f='possible' AND (partial2.account IS NULL OR partial2.account='') THEN
						result.rtp_forward := 'yes';
					END IF;
						result.called:=newcalled;
					RETURN next result;
					ips[dim] := partial2.ip;
					dim := dim+1;
					IF iteration>2 THEN
						EXIT;
					END IF;
				END IF;
				iteration3 := iteration3+1;
			END LOOP;
		END LOOP;
		RETURN;
	END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION public.getroute(text, text) OWNER TO postgres;

--
-- Name: initaccounts(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION initaccounts() RETURNS SETOF record
    AS $$
	DECLARE
		result record;
	BEGIN
		FOR result IN SELECT enabled ,protocol,username, description,interval,formats,authname ,password ,server ,
gatekeeper,domain,registrar,outbound ,localaddress, modified, account ,id,status,number,TRUE AS gw FROM accounts WHERE enabled IS TRUE AND account IS NOT NULL AND username IS NOT NULL ORDER BY account LOOP
			RETURN next result;
		END LOOP;
		RETURN;
	END;
$$
    LANGUAGE plpgsql;


ALTER FUNCTION public.initaccounts() OWNER TO postgres;

--
-- Name: rewritenr(text, boolean, smallint, smallint, boolean, smallint, text, smallint); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION rewritenr(text, boolean, smallint, smallint, boolean, smallint, text, smallint) RETURNS text
    AS $_$
	DECLARE
		called ALIAS FOR $1;
		cutbool ALIAS FOR $2;
		digitnr ALIAS FOR $3;
		poscut ALIAS FOR $4;
		replacebool ALIAS FOR $5;
		posrpl ALIAS FOR $6;
		digitrplnr ALIAS FOR $8;
		digitrpl ALIAS FOR $7;
		front text;
		back text;
		result text;
	BEGIN

		result:=called;
		IF cutbool IS TRUE THEN
			result:=substr(result,1,poscut-1) || substr(result,poscut+digitnr,length(result)-poscut-digitnr+1);
		ELSEIF replacebool IS TRUE THEN
			IF digitrplnr IS NULL THEN
				RETURN digitrpl;
			END IF;
			result:=substr(result,1,posrpl-1) || digitrpl || substr(result,posrpl+digitrplnr,length(result)-posrpl-digitrplnr+1);
		END IF;
		RETURN result;
	END;
$_$
    LANGUAGE plpgsql;


ALTER FUNCTION public.rewritenr(text, boolean, smallint, smallint, boolean, smallint, text, smallint) OWNER TO postgres;

--
-- Name: unregister(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION unregister() RETURNS SETOF record
    AS $$
	DECLARE
		result record;
	BEGIN
		FOR result IN SELECT line FROM lines WHERE expires<CURRENT_TIMESTAMP LOOP
			UPDATE lines SET expires=NULL,location=NULL WHERE line=result.line;
			RETURN next result;
		END LOOP;
		RETURN;
	END;
$$
    LANGUAGE plpgsql;


--
-- Name: updateaccounts(); Type: FUNCTION; Schema: public; Owner: postgres
--

CREATE FUNCTION updateaccounts() RETURNS SETOF record
    AS $$
	DECLARE
		result record;
	BEGIN
		FOR result IN SELECT enabled ,protocol,username, description,interval,formats,authname ,password ,server ,
gatekeeper,domain,registrar,outbound ,localaddress, modified, account ,id, status, number,TRUE AS gw FROM accounts WHERE enabled IS TRUE AND modified IS TRUE LOOP
			RETURN next result;
		END LOOP;
		UPDATE accounts SET modified=false WHERE modified IS TRUE;
		RETURN;
	END;

$$
    LANGUAGE plpgsql;


ALTER FUNCTION public.updateaccounts() OWNER TO postgres;

SET default_tablespace = '';

SET default_with_oids = true;

--
-- Name: accounts; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE accounts (
    enabled boolean,
    protocol text,
    username text,
    description text,
    "interval" text,
    formats text,
    authname text,
    "password" text,
    number text,
    server text,
    gatekeeper text,
    "domain" text,
    registrar text,
    outbound text,
    localaddress text,
    modified boolean,
    account text,
    id serial NOT NULL,
    status text
);


ALTER TABLE public.accounts OWNER TO postgres;

--
-- Name: accounts_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--

SELECT pg_catalog.setval(pg_catalog.pg_get_serial_sequence('accounts', 'id'), 1, true);


SET default_with_oids = false;

--
-- Name: cdr; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE cdr (
    cid bigserial,
    "time" TIMESTAMP with time zone NOT NULL,
    channel text,
    address text,
    direction text,
    billid text,
    caller text,
    "called" text,
    duration time without time zone,
    ringtime time without time zone,
    billtime time without time zone,
    status text,
    reason text,
    ended boolean
);


ALTER TABLE public.cdr OWNER TO postgres;

SET default_with_oids = true;

--
-- Name: form; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE form (
    "location" text,
    "called" text,
    error text,
    line text,
    formats text,
    rtp_forward text,
    divert_busy text,
    divert_noanswer text,
    maxcall integer
);


ALTER TABLE public.form OWNER TO postgres;

--
-- Name: lines; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE lines (
    line text,
    "password" text,
    first_name text,
    last_name text,
    address text,
    inuse integer,
    "location" text,
    expires timestamp without time zone
);


ALTER TABLE public.lines OWNER TO postgres;

--
-- Name: messages; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE messages (
    message text,
    date timestamp without time zone
);


ALTER TABLE public.messages OWNER TO postgres;

--
-- Name: pbx; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE pbx (
    line text,
    param text,
    value text
);


ALTER TABLE public.pbx OWNER TO postgres;

--
-- Name: routes; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE routes (
    priority smallint,
    protocol text,
    prefix text,
    ip text,
    port text,
    iaxuser text,
    iaxcontext text,
    chan_group text,
    id serial NOT NULL,
    account text,
    cutbool boolean,
    nrdigcut smallint,
    poscut smallint,
    replacebool boolean,
    posrpl smallint,
    digrpl text,
    nrdigrpl smallint,
    formats text,
    rtp_forward boolean
);


ALTER TABLE public.routes OWNER TO postgres;

--
-- Name: routes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: postgres
--

SELECT pg_catalog.setval(pg_catalog.pg_get_serial_sequence('routes', 'id'), 1, true);


--
-- Name: settings; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE settings (
    param text,
    value text
);


ALTER TABLE public.settings OWNER TO postgres;

--
-- Name: users; Type: TABLE; Schema: public; Owner: postgres; Tablespace:
--

CREATE TABLE users (
    username text,
    "password" text,
    "first" text,
    "last" text,
    description text
);


ALTER TABLE public.users OWNER TO postgres;

--
-- Data for Name: form; Type: TABLE DATA; Schema: public; Owner: postgres
--

COPY form ("location", "called", error, line, formats, rtp_forward, divert_busy, divert_noanswer, maxcall) FROM stdin;
none	\N	\N	\N	\N	\N	\N	\N	\N
none	\N	\N	\N	\N	\N	\N	\N	\N
none	\N	\N	\N	\N	\N	\N	\N	\N
\.


--
-- Data for Name: settings; Type: TABLE DATA; Schema: public; Owner: postgres
--

COPY settings (param, value) FROM stdin;
version	1
vm	external/nodata/leavemail.php
\.


--
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: postgres
--

COPY users (username, "password", "first", "last", description) FROM stdin;
admin	admin	admin	admin	admin
\.


--
-- Name: line_index; Type: INDEX; Schema: public; Owner: postgres; Tablespace:
--

CREATE INDEX line_index ON pbx USING btree (line);


ALTER INDEX public.line_index OWNER TO postgres;

--
-- Name: timeindex; Type: INDEX; Schema: public; Owner: postgres; Tablespace:
--

CREATE INDEX timeindex ON cdr USING btree ("time");


ALTER INDEX public.timeindex OWNER TO postgres;

--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

